#include "stdafx.h"
#include "COM.h"
#include "Setter.h"

using namespace System::IO::Ports;
using namespace System::Windows::Forms;

COM::COM(String ^port, int baud) { 
	this->port = gcnew SerialPort(port, baud, Parity::None, 8, StopBits::One);
	this->port->ReadTimeout = 1000;
	this->port->WriteTimeout = 1000;
	try {
		this->port->Open();
	} catch (Exception ^ex) {
		throw gcnew COMException(L"COM Error", L"The COM port could not be opened:\n"+ex->Message);
	}
	if (!this->port->IsOpen) {
		throw gcnew COMException(L"COM Error", L"The COM port could not be opened");
	}
	this->port->DiscardInBuffer();
	stream = this->port->BaseStream;
}
void COM::Close() { if (port) { port->Close(); port = nullptr; stream = nullptr; } }
COM::~COM() { Close(); }

void COM::SendCmd(const Byte cmd) { stream->WriteByte(254); stream->WriteByte(cmd); stream->Flush(); }
void COM::SendCmd(const Byte cmd, const int subCmd) {
	stream->WriteByte(254);
	stream->WriteByte(cmd);
	if (subCmd >= 0 && subCmd < 256) stream->WriteByte(subCmd);
	stream->Flush();
}
void COM::SendCmd(const Byte cmd, const Byte param1, const Byte param2) {
	stream->WriteByte(254); stream->WriteByte(cmd); stream->WriteByte(param1); stream->WriteByte(param2); stream->Flush();
}
int COM::ReadValue(const Byte cmd) { port->DiscardInBuffer(); SendCmd(cmd); return port->ReadByte(); }
int COM::ReadValue(const Byte cmd, const int subCmd) { port->DiscardInBuffer(); SendCmd(cmd, subCmd); return port->ReadByte(); }
unsigned short COM::ReadShort(const Byte cmd) { port->DiscardInBuffer(); SendCmd(cmd); return port->ReadByte() | port->ReadByte() << 8; }
array<Byte> ^COM::ReadBytes(const Byte cmd, const int n) { return ReadBytes(cmd, -1, n); }
array<Byte> ^COM::ReadBytes(const Byte cmd, const int subCmd, const int n) {
	port->DiscardInBuffer();
	SendCmd(cmd, subCmd);
	array<Byte> ^buffer = gcnew array<Byte>(n+1);
	System::Threading::Thread::Sleep(100); // make sure the PIC has time to process this and start sending data
	int read = port->Read(buffer, 0, n);
	while (read < n)
		read += port->Read(buffer, read, n-read);
	buffer[n] = 0;
	return buffer;
}
void COM::SendCharacter(const Byte cmd, const Byte c, array<Byte> ^bytes) {
	SendCmd(cmd, c);
	stream->Write(bytes, 0, 8);
	stream->Flush();
}
void COM::SendLine(String ^line, bool cmdEsc) {
	line = line->PadRight(20);
	for (int i = 0; i < 20; i++) {
		if (line[i] == 0x100)
			stream->WriteByte(0); // due to not able to have 0 in a string...
		else if (line[i] > 0xFF)
			stream->WriteByte(32); // space
		else if (cmdEsc && line[i] == 254) { // escape the 254 character with another 254
			stream->WriteByte(254); stream->WriteByte(254);
		} else
			stream->WriteByte((unsigned char)line[i]);
	}
	stream->Flush();
}
void COM::ReadGPOVal(RadioButton ^on, RadioButton ^off, TrackBar ^bar, Label ^label, int i, bool cur) {
	int gpo = ReadValue(cur ? ReadGPO : ReadSavedGPO, i);
	int pwm = ReadValue(cur ? ReadGPOpwm : ReadSavedGPOpwm, i);
	on->Checked = gpo==1;
	off->Checked = gpo==0;
	bar->Value = pwm;
	label->Text = pwm.ToString();
}

COMException::COMException(String ^title, String ^text) : title(title), text(text), Exception(text) {}

void COMException::Show(Form ^f) {
	if (f->GetType() == lcdsetter::Setter::typeid) {
		((lcdsetter::Setter^)f)->ShowErrorMessage(text, title);
	} else {
		MessageBox::Show(f, text, title, MessageBoxButtons::OK, MessageBoxIcon::Error);
	}
}
