#pragma once

using namespace System;

/**
A simple interface for a COM port.
This is really just a wrapper for the .NET System::IO::Ports::SerialPort.
It handles sending commands and reading data.
**/

ref class COM {
public:
	System::IO::Ports::SerialPort ^port;
	System::IO::Stream ^stream;

	COM(String ^port, int baud);
	void Close(); // Closes the underlying serial port
	~COM();	// Calls Close()

	void SendCmd(const Byte cmd); // Send a command with no parameters
	void SendCmd(const Byte cmd, const int subCmd); // Send a command that takes a single parameter
	void SendCmd(const Byte cmd, const Byte param1, const Byte param2); // Send a command that takes 2 params
	int ReadValue(const Byte cmd); // Send a command and read the returned byte
	int ReadValue(const Byte cmd, const int subCmd); // Send a command with a parameter and read the returned byte
	unsigned short ReadShort(const Byte cmd); // Send a command and read the returned 2 bytes
	array<Byte> ^ReadBytes(const Byte cmd, const int n); // Send a command and read n returned bytes
	array<Byte> ^ReadBytes(const Byte cmd, const int subCmd, const int n); // Send a command with a parameter and read n returned bytes

	void SendCharacter(const Byte cmd, const Byte c, array<Byte> ^bytes); // Send an entire character using cmd (DefineCustom or RememberCustom) and a character #0-7
	void SendLine(String ^line, bool cmdEsc); // Send a line of data, may need a Position or SaveStartup command before this, will force line to 20 characters with spaces

	// Reads a GPO val
	void ReadGPOVal(System::Windows::Forms::RadioButton ^on, System::Windows::Forms::RadioButton ^off, System::Windows::Forms::TrackBar ^bar, System::Windows::Forms::Label ^label, int i, bool cur);
};

// A special excpetion class thrown by the COM constructor that has a title and text ready for a dialog box
ref class COMException : Exception {
public:
	String ^title;
	String ^text;
	COMException(String ^title, String ^text);
	// Shows a dialog box with the title of this exception and the text of this exception
	void Show(System::Windows::Forms::Form ^f);
};