#include "StdAfx.h"
#include "CustomChar.h"

using namespace System;
using namespace System::Windows::Forms;
using namespace System::Drawing;
using namespace lcdsetter;

#define CC_WIDTH 5
CustomChar::CustomChar(void) {
	active_r = active_c = -1;
	data = gcnew array<Byte>(8);
	data[0] = 1;
	data[1] = 2;
	data[2] = 4;
	data[3] = 8;
	data[4] = 16;
	data[5] = 8;
	data[6] = 4;
	data[7] = 2;
	this->SuspendLayout();
	this->DoubleBuffered = true;
	this->SetStyle(ControlStyles::FixedWidth, true);
	this->SetStyle(ControlStyles::FixedHeight, true);
	this->SetStyle(ControlStyles::Selectable, true);
	this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::None;
	this->BackColor = System::Drawing::Color::White;
	this->Cursor = System::Windows::Forms::Cursors::Cross;
	this->ForeColor = System::Drawing::Color::Black;
	this->MaximumSize = System::Drawing::Size(CC_WIDTH*10, 80);
	this->MinimumSize = System::Drawing::Size(CC_WIDTH*10, 80);
	this->Name = L"CustomChar";
	this->Size = System::Drawing::Size(CC_WIDTH*10, 80);
	this->ResumeLayout(false);
}
array<Byte> ^CustomChar::GetDataDynamic() {
	return data;
}
array<Byte> ^CustomChar::GetData() {
	array<Byte> ^arr = gcnew array<Byte>(8);
	System::Array::Copy(data, arr, 8);
	return arr;
}
Byte CustomChar::GetRowData(int row) {
	return data[row];
}
bool CustomChar::GetPixelData(int row, int col) {
	return 0x1 & (data[row] >> (CC_WIDTH-col-1));
}
void CustomChar::SetData(array<Byte> ^rows) {
	System::Array::Copy(rows, data, 8);
	Invalidate();
}
void CustomChar::SetRowData(int row, Byte value) {
	data[row] = value;
	Invalidate();
}
void CustomChar::SetPixelData(int r, int c, bool on) {
	c = CC_WIDTH-c-1;
	if (on)	data[r] |= 1 << c;
	else	data[r] &= ~(1 << c);
	Invalidate();
}
void CustomChar::InvertPixel(int r, int c) {
	bool x = GetPixelData(r, c);
	c = CC_WIDTH-c-1;
	if (!x)	data[r] |= 1 << c;
	else	data[r] &= ~(1 << c);
	Invalidate();
}
void CustomChar::OnPaint(System::Windows::Forms::PaintEventArgs^ e ) {
	//System::Windows::Forms::UserControl::OnPaint(e);
	System::Drawing::Graphics ^g = e->Graphics;
	System::Drawing::Brush ^bg = gcnew System::Drawing::SolidBrush(this->Enabled ? BackColor : SystemColors::Control);
	System::Drawing::Brush ^fg = gcnew System::Drawing::SolidBrush(this->Enabled ? ForeColor : SystemColors::GrayText);
	g->FillRectangle(bg, 0, 0, CC_WIDTH*10, 80);
	for (int y = 0; y < 8; y++)
		for (int x = 0; x < CC_WIDTH; x++) {
			bool on = (data[y] >> (CC_WIDTH-x-1)) & 0x1;
			if (this->Enabled && y == active_r && x == active_c) { // highlight pixel
				if (on) 
					g->FillRectangle(Brushes::Green, x*10, y*10, 10, 10); 
				else
					g->FillRectangle(Brushes::GreenYellow, x*10, y*10, 10, 10);
			} else if (on) {
				g->FillRectangle(fg, x*10, y*10, 10, 10);
			}
		}
	if (this->Focused && this->Enabled) { // draw focus rectangle
		System::Drawing::Pen ^p = gcnew System::Drawing::Pen(fg, 1);
		array<float> ^temp0 = {1.0F,1.0F};
		p->DashPattern = temp0;
		g->DrawRectangle(p, 0, 0, CC_WIDTH*10-1, 79);
		delete p;
	}
	delete bg;
	delete fg;
}
void CustomChar::OnGotFocus(System::EventArgs^ e) { Invalidate(); UserControl::OnGotFocus(e); } //needed to redraw border
void CustomChar::OnLostFocus(System::EventArgs^ e) { Invalidate(); UserControl::OnLostFocus(e); }
void CustomChar::OnEnabledChanged(System::EventArgs ^e) { Invalidate(); UserControl::OnEnabledChanged(e); } //needed to change colors to disabled
void CustomChar::OnMouseClick(System::Windows::Forms::MouseEventArgs ^e) {
	if (this->Enabled && e->Button == Windows::Forms::MouseButtons::Right) {
		// show data
		MessageBox::Show(this, data[0]+L","+data[1]+L","+data[2]+L","+data[3]+L","+data[4]+L","+data[5]+L","+data[6]+L","+data[7],"Custom Character Data");
	} else if (this->Enabled) {
		// invert a pixel
		int c = e->X / 10;
		int r = e->Y / 10;
		if (c < CC_WIDTH && r < 8 && c >= 0 && r >= 0)
			InvertPixel(r, c);
	}
	UserControl::OnMouseClick(e);
}
void CustomChar::OnMouseLeave(System::EventArgs ^e) {
	// during of highlighting
	active_c = -1;
	active_r = -1;
	Invalidate();
	UserControl::OnMouseLeave(e);
}
void CustomChar::OnMouseMove(System::Windows::Forms::MouseEventArgs ^e) {
	// highlight a pixel
	int c = e->X / 10;
	int r = e->Y / 10;
	if (c < CC_WIDTH && r < 8 && c >= 0 && r >= 0) {
		active_c = c;
		active_r = r;
	} else {
		active_c = -1;
		active_r = -1;
	}
	Invalidate();
	UserControl::OnMouseMove(e);
}