#include "stdafx.h"
#include "DeviceChooser.h"
#include "COM.h"
#include "Setter.h"

#define MAX_ITEMS 32

using namespace System;
using namespace System::Windows::Forms;
using namespace System::IO::Ports;

using namespace lcdsetter;

DeviceChooser::DeviceChooser(void) {
	InitializeComponent();
	RefreshList();
}

void DeviceChooser::RefreshList() {
	combo->Items->Clear();

	int coms[MAX_ITEMS];
	int bls[MAX_ITEMS];

	int count_coms = getUSB2LCD_com(coms, MAX_ITEMS);
	int count_bls = getUSB2LCD_bl(bls, MAX_ITEMS);

	if (count_coms < 0 && count_coms != BUFFER_OVERFLOW) {
		MessageBox::Show(this, L"Could not get the number of devices in COM mode (Error: "+count_coms+L")", L"Device Search Error", MessageBoxButtons::OK, MessageBoxIcon::Warning);
		count_coms = 0;
	}
	if (count_bls == BUFFER_OVERFLOW || count_coms == BUFFER_OVERFLOW) {
		MessageBox::Show(this, L"Only "+MAX_ITEMS+L" devices in COM or bootloader mode are available here even though more are plugged in.", L"Too Many Devices", MessageBoxButtons::OK, MessageBoxIcon::Warning);
		if (count_bls == BUFFER_OVERFLOW) count_bls = MAX_ITEMS;
		if (count_coms == BUFFER_OVERFLOW) count_coms = MAX_ITEMS;
	}

	for (int i = 0; i < count_bls; i++) {
		combo->Items->Add(gcnew Device(BOOTLOADER, bls[i], getVersion_bl(bls[i])));
	}
	for (int i = 0; i < count_coms; i++) {
		combo->Items->Add(gcnew Device(U2L_COM, coms[i], getVersion_com(coms[i])));
	}

	for each (String ^s in SerialPort::GetPortNames()) {
		for (int i = s->Length-1; i >= 3; i--) {
			if (Char::IsDigit(s[i])) {
				s = s->Substring(3, i-2);
				break;
			}
		}
		int id = Convert::ToInt32(s), i;
		for (i = 0; i < count_coms; i++) {
			if (coms[i] == id) break;
		}
		union VersionInfo v;
		v._all = 0;
		if (i == count_coms)
			combo->Items->Add(gcnew Device(COM_DEVICE, id, v));
	}

	if (combo->Items->Count == 0) {
		combo->Items->Add(L"No devices found");
		buttonSelect->Enabled = false;
	} else {
		buttonSelect->Enabled = true;
	}

	combo->SelectedIndex = 0;
}
void DeviceChooser::buttonRefresh_Click(System::Object^ sender, System::EventArgs^ e) {
	this->RefreshList();
}
void DeviceChooser::buttonSelect_Click(System::Object^  sender, System::EventArgs^ e) {
	Device ^d = (Device^)combo->SelectedItem;
	this->DialogResult = System::Windows::Forms::DialogResult::OK;
	this->device = d;
	this->Close();
}

VersionInfo DeviceChooser::getVersion_com(int port) {
	COM ^c = nullptr;
	union VersionInfo v;
	v._all = 0;
	try {
		c = gcnew COM(L"COM"+port, 9600);
		v.version = c->ReadValue(ReadVersion);
		v.module = c->ReadValue(ReadModuleType);
		v.serialnumber = c->ReadShort(ReadSerialNum);;
		c->Close();
	} catch (COMException ^ex) {
		ex->Show(this);
	} catch (Exception ^ex) {
		MessageBox::Show(this, L"The COM port caused an error:\n"+ex->Message, L"COM Error", MessageBoxButtons::OK, MessageBoxIcon::Error);
	} finally { if (c) c->Close(); }
	return v;
}

void DeviceChooser::InitializeComponent() {
	System::Windows::Forms::Label^  label1;
	System::ComponentModel::ComponentResourceManager^  resources = (gcnew System::ComponentModel::ComponentResourceManager(DeviceChooser::typeid));
	this->combo = (gcnew System::Windows::Forms::ComboBox());
	this->buttonRefresh = (gcnew System::Windows::Forms::Button());
	this->buttonSelect = (gcnew System::Windows::Forms::Button());
	label1 = (gcnew System::Windows::Forms::Label());
	this->SuspendLayout();
	// 
	// label1
	// 
	label1->AutoSize = true;
	label1->Location = System::Drawing::Point(12, 9);
	label1->Text = L"Select a device to use:";
	// 
	// combo
	// 
	this->combo->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left) 
		| System::Windows::Forms::AnchorStyles::Right));
	this->combo->DropDownStyle = System::Windows::Forms::ComboBoxStyle::DropDownList;
	this->combo->FormattingEnabled = true;
	this->combo->Location = System::Drawing::Point(15, 30);
	this->combo->Size = System::Drawing::Size(250, 21);
	this->combo->TabIndex = 0;
	// 
	// buttonRefresh
	// 
	this->buttonRefresh->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Right));
	this->buttonRefresh->Location = System::Drawing::Point(271, 29);
	this->buttonRefresh->Size = System::Drawing::Size(75, 23);
	this->buttonRefresh->TabIndex = 1;
	this->buttonRefresh->Text = L"Refresh";
	this->buttonRefresh->UseVisualStyleBackColor = true;
	this->buttonRefresh->Click += gcnew System::EventHandler(this, &DeviceChooser::buttonRefresh_Click);
	// 
	// buttonSelect
	// 
	this->buttonSelect->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Right));
	this->buttonSelect->Location = System::Drawing::Point(352, 29);
	this->buttonSelect->Size = System::Drawing::Size(75, 23);
	this->buttonSelect->TabIndex = 2;
	this->buttonSelect->Text = L"Select";
	this->buttonSelect->UseVisualStyleBackColor = true;
	this->buttonSelect->Click += gcnew System::EventHandler(this, &DeviceChooser::buttonSelect_Click);
	// 
	// DeviceChooser
	// 
	this->AcceptButton = this->buttonSelect;
	this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
	this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
	this->ClientSize = System::Drawing::Size(439, 64);
	this->Controls->Add(label1);
	this->Controls->Add(this->buttonSelect);
	this->Controls->Add(this->buttonRefresh);
	this->Controls->Add(this->combo);
	this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedDialog;
	this->Icon = (cli::safe_cast<System::Drawing::Icon^  >(resources->GetObject(L"$this.Icon")));
	this->MaximizeBox = false;
	this->Name = L"DeviceChooser";
	this->Text = L"LCD Setter: Choose Device";
	this->ResumeLayout(false);
	this->PerformLayout();
}