#include "StdAfx.h"
#include "LCDText.h"

using namespace lcdsetter;

void LCDText::BuildCharMap() {
	FILE *f = fopen("chars.txt", "rS,ccs=UNICODE");
	if (f == NULL) {
		char_map = char_map_default; // use defaults
	} else {
		char_map = gcnew array<wchar_t>(256);
		bool in_comment = false;
		int index = 0;
		while (!feof(f) && index < 256) {
			wchar_t c = fgetwc(f);
			if (c == L'\t') {
				in_comment = true;
			} else if (c == L'\r' || c == L'\n') {
				in_comment = false;
			} else if (!in_comment && c >= L' ') {
				char_map[index++] = c;
			}
		}
		while (index < 256)
			char_map[index++] = L' ';
		fclose(f);
	}
}

wchar_t LCDText::GetDisplayChar(wchar_t c) {
	if (char_map == nullptr) { BuildCharMap(); }

	return char_map[c];
}

System::String ^LCDText::GetDisplayText(System::String ^text) {
	if (char_map == nullptr) { BuildCharMap(); }

	wchar_t *newtext = new wchar_t[text->Length];
	for (int i = 0; i < text->Length; i++) {
		wchar_t cur = text[i];
		if (cur == 0x100) cur = 0;
		if (cur > 126)	newtext[i] = cur;
		else			newtext[i] = char_map[cur];
	}
	text = gcnew System::String(newtext, 0, text->Length);
	delete[] newtext;
	return text;
}

System::String ^LCDText::UnGetDisplayText(System::String ^text) {
	if (char_map == nullptr) { BuildCharMap(); }
	if (char_map_ == nullptr) {
		char_map_ = gcnew System::Collections::Generic::Dictionary<wchar_t, wchar_t>();
		for (wchar_t i = 0; i < char_map->Length; i++) {
			char_map_[char_map[i]] = i?i:0x100;
		}
	}

	wchar_t *newtext = new wchar_t[text->Length];
	for (int i = 0; i < text->Length; i++) {
		wchar_t cur = text[i];
		if (cur > 126)
			newtext[i] = char_map_[cur];
		else
			newtext[i] = cur;
	}
	text = gcnew System::String(newtext, 0, text->Length);
	delete[] newtext;
	return text;
}

LCDText::LCDText(void) : System::Windows::Forms::TextBox() {
	this->SuspendLayout();
	this->Font = gcnew System::Drawing::Font(L"Courier New", 12.0F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, 0);
	this->HideSelection = false;
	this->MaxLength = 20;
	this->ResumeLayout(false);
}

void LCDText::Text::set(System::String ^x) { System::Windows::Forms::TextBox::Text::set(x->Replace(L'\t', L' ')); } // removes tab characters
System::String ^LCDText::RealText::get() { return UnGetDisplayText(this->Text); }
void LCDText::RealText::set(System::String ^x) { this->Text = GetDisplayText(x); }
