#pragma once

/*
A user control to extend the abilities of the text box.
It maps all displayed characters to some other characters for display
But all characters can be mapped back
Uses the chars.txt file if available
*/
namespace lcdsetter {
	public ref class LCDText : public System::Windows::Forms::TextBox {
	public:
		// Gets the actually character to display as
		static wchar_t GetDisplayChar(wchar_t c);

		// Converts a whole string to the actual display characters
		static System::String ^GetDisplayText(System::String ^text);

		// Converts a display string back to the real characters to write to the LCD
		static System::String ^UnGetDisplayText(System::String ^text);

		// Overriden to convert tabs to spaces
		property System::String ^Text { virtual void set(System::String ^x) override; };

		// Gets the real (not display) text for this text box.
		property System::String ^RealText { System::String ^get(); void set(System::String ^x); };

		LCDText(void);
	private:
		static void BuildCharMap(); // builds the char_map variable
		static array<wchar_t> ^char_map = nullptr; // the mapping of real (0-255) to display (any valid UNICODE) characters
		static System::Collections::Generic::Dictionary<wchar_t, wchar_t> ^char_map_ = nullptr; // the reverse char map (display to real)
		// the default char_map when no chars.txt file is given
		static array<wchar_t> ^char_map_default = {
			0xFF11, 0xFF12, 0xFF13, 0xFF14, 0xFF15, 0xFF16, 0xFF17, 0xFF18, // custom characters 1-8 (glyphs are poorly drawn numbers)
			0x2581, 0x2582, 0x2583, 0x2584, 0x2585, 0x2586, 0x2587, 0x2588, // custom characters 1-8 repeat (glyphs are increasing box sizes, could just use the above again)
			0xB1, 0x2261, //+/-, identity (triple equal)
			0x2196, 0x2199, // sigma parts
			0x256D, 0x2570, // left parenthesis parts
			0x256E, 0x256F, // right parenthesis parts
			0x283, 0x285, // curly brace parts
			0x2248, 0x222B , 0x2017, 0x219C, // congruent (2 squiggles), integral, double low line, squiggle
			0xB2, 0xB3, // squared, cubed
			32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, // space and symbols
			48, 49, 50, 51, 52, 53, 54, 55, 56, 57, //0-9
			58, 59, 60, 61, 62, 63, 64, // symbols
			65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, // A-Z
			91, 92, 93, 94, 95, 96, // symbols
			97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, // a-z
			123, 124, 125, 126, // symbols
			0x2206, // delta
			0xC7, // C w/ cedilla
			0xFC, // u w/ diaeresis
			0xE9, // e w/ acute
			0xE2, 0xE4, 0xE0, 0xE5, // a w/ circumflex, diaeresis, grave, ring
			0xE7, // c w/ cedilla
			0xEA, 0xEB, 0xE8, // e w/ circumflex, diaeresis, grave
			0xEF, 0xEE, 0xEC, // i w/ diaeresis, circumflex, grave
			0xC4, 0xC2, // A w/ diaeresis, circumflex
			0xC9, // E w/ acute
			0xE6, 0xC6, // ae, AE
			0xF4, 0xF6, 0xF2, // o w/ circumflex, diaeresis, grave
			0xFB, 0xF9, // u w/ circumflex, grave
			0xFF, 0xD6, 0xDC, // y/O/U w/ diaeresis
			0xF1, 0xD1, // n/N w/ tilde
			0x101, 0x14D, // a/o with underline (the glyph has it above though...)
			0xBF, // inverted ?
			0xE1, 0xED, 0xF3, 0xFA, // a/i/o/u w/ accute
			0xA2, 0xA3, 0xA5, 0x20A7, 0x192, // cents, pounds, yen, Pts (Spanish currency), cursive f (florin, currency of netherlands / aruba)
			0xA1, // inverted !
			0xC3, 0xE3, //A/a w/ tilde
			0xD5, 0xF5, //O/o w/ tilde
			0xD8, 0xF8, // O/o w/ stroke
			0x2D9, 0xA8, 0x2DA, 0x2CB, 0x2CA, //dot, diaeresis, ring, grave, acute
			0xBD, 0xBC, // 1/2 and 1/4
			0xD7, 0xF7, 0x2264, 0x2265, //multiply, divide, less than equal, greater than equal
			0xAB, 0xBB, // double angle brackets
			0x2260, 0x221A, 0xAF, // not equal, square root, overline (macron)
			0x2320, 0x2321, // integral half symbols
			0x221E, // infinity
			0x21D6, 0x21B5, // triangle in corner (glyph is a double arrow to NW), new line
			0x2191, 0x2193, 0x2192, 0x2190, // arrows up, down, right, left
			0x250C, 0x2510, 0x2514, 0x2518, // corners: tl, tr, bl, br
			0x2022, 0xAE, 0xA9, 0x2122, // bullet, registered, copyright, trademark
			0x2020, 0xA7, 0xB6, // dagger, section, paragraph (pilcrow)
			0x393, 0x394, 0x398, 0x39B, 0x39E, 0x3A0, 0x3A3, 0x3A5, 0x3A6, 0x3A8, 0x3A9, //Gamma, Delta?, Theta, Lamda, Xi, Pi, Sigma, Upsilon?, Phi, Psi, Omega
			0x3B1, 0x3B2, 0x3B3, 0x3B4, 0x3B5, 0x3B6, 0x3B7, 0x3B8, 0x3B9, 0x3BA, 0x3BB, 0x3BC, 0x3BD, 0x3BE, 0x3C0, 0x3C1, 0x3C3, 0x3C4, 0x3C5, 0x3C7, 0x3C8, 0x3C9, //alpha-omega, skips omicron and phi
			0x25BC, 0x25BA, 0x25C4, // black arrow down, right, left
			0x211C, 0x21A4, 0x2131, 0x21E5, // big R (glyph is cursive), left arrow from , big F (glyph is cursive), right arrow to bar
			0x25A1, 0x25AC, // white square, black rectangle (thick hyphen)
			0x282, 0x2119 // s in box (glyph is just an S with a hook), p in box (glyph is double-stuck P)
		};
	};
}
