#include "StdAfx.h"
#include "Setter.h"
#include "Dialog.h"

using namespace lcdsetter;

Setter::Setter(bool isU2L, int port) {
	this->isU2L = isU2L;
	this->port = L"COM"+port;

	InitializeComponent();
	lastLine = textLine1;
	customChar1->SetData(one);
	customChar2->SetData(two);
	customChar3->SetData(three);
	customChar4->SetData(four);
	customChar5->SetData(five);
	customChar6->SetData(six);
	customChar7->SetData(seven);
	customChar8->SetData(eight);

	UpdateVersionInfo();

	butReadMem->Enabled = isU2L;
	butReadCur->Enabled = isU2L;
	butReadButtons->Enabled = isU2L;
	butFirmware->Enabled = isU2L;
}
void Setter::UpdateVersionInfo() {
	union VersionInfo v = GetVersion();
	if (isU2L) {
		this->labelPort->Text = L"USB2LCD+ Device: "+port+", v"+DISP_VERSION(v.version)+L", ID: "+v.serialnumber.ToString("X4");
	} else {
		this->labelPort->Text = L"Other Device: "+port+", module type: "+v.module.ToString("X2")+", v"+v.version.ToString("X2")+L", ID: "+v.serialnumber.ToString("X4");
	}
}
void Setter::ShowMessage(String ^msg, String ^title, MessageBoxButtons but, MessageBoxIcon icon) {
	if (this->InvokeRequired) {
		this->Invoke(gcnew ShowMessage$(this, &Setter::ShowMessage), msg, title, but, icon);
	} else {
		MessageBox::Show(this, msg, title, but, icon);
	}
}
void Setter::ShowMessage(String ^msg, String ^title, MessageBoxIcon icon) { ShowMessage(msg, title, MessageBoxButtons::OK, icon); }
void Setter::ShowErrorMessage(String ^msg, String ^title) { ShowMessage(msg, title, MessageBoxButtons::OK, MessageBoxIcon::Error); }
DialogResult Setter::ShowFileDialog() {
	if (this->InvokeRequired) {
		return (System::Windows::Forms::DialogResult)this->Invoke(gcnew ShowDialog$(openFileDialog, &OpenFileDialog::ShowDialog), this);
	} else {
		return openFileDialog->ShowDialog(this) ;
	}
}
void Setter::EnableForm(bool e) {
	if (this->InvokeRequired) {
		this->Invoke(gcnew EnableForm$(this, &Setter::EnableForm), e);
	} else {
		this->Cursor = e ? Cursors::Default : Cursors::WaitCursor;
		butReadMem->Enabled = e && isU2L;
		butReadCur->Enabled = e && isU2L;
		butReadButtons->Enabled = e && isU2L;
		butFirmware->Enabled = e && isU2L;
		butSetCur->Enabled = e;
		butClear->Enabled = e;
		butSetMem->Enabled = e;
		butGPO->Enabled = e;
		groupDisplay->Enabled = e;
		groupScreen->Enabled = e;
		groupCustom->Enabled = e;
		groupMessage->Enabled = e;
	}
}

VersionInfo Setter::GetVersion() {
	COM ^c = nullptr;
	union VersionInfo v;
	v._all = 0;
	try {
		c = gcnew COM(port, BAUD);
		v.version = c->ReadValue(ReadVersion);
		v.module = c->ReadValue(ReadModuleType);
		v.serialnumber = c->ReadShort(ReadSerialNum);
		c->Close();
	} catch (COMException ^ex) {
		ex->Show(this);
	} catch (Exception ^ex) {
		ShowErrorMessage(L"The COM port caused an error:\n"+ex->Message, L"COM Error");
	} finally { if (c) c->Close(); }
	return v;
}

#define MAX_ITEMS 32

int findDifference(bool unique, int serial, int *a, int alen, int *b, int blen) {
	if (unique) {
		for (int i = 0; i < blen; i++) {
			if (getVersion_bl(b[i]).serialnumber == serial) {
				return b[i];
			}
		}
	} else {
		int len = alen < blen ? alen : blen;
		for (int i = 0; i < len; i++) {
			if (a[i] != b[i]) { return b[i]; }
		}
		if (blen > alen) { return b[len]; }
	}
	return -1;
}
void Setter::UpdateFirmware() {
	// get the HEX file to use
	wchar_t *fileName;
	if (ShowFileDialog() == System::Windows::Forms::DialogResult::OK) {
		fileName = toWCS(openFileDialog->FileName);
		int retval = checkHEXfile(fileName);
		if (retval < 0) {
			ShowErrorMessage(L"The selected file could not be read / parsed: "+retval, L"File Error");
			delete[] fileName;
			EnableForm(true);
			return;
		}
	} else {
		EnableForm(true);
		return;
	}

	this->BeginInvoke(gcnew DialogShowModal$(Dialog::ShowModal), this, L"Firmware Update", L"Please wait while the device resets...");

	int ids[MAX_ITEMS];
	int count = getUSB2LCD_bl(ids, MAX_ITEMS);
	unsigned short serial;
	bool unique = true;

	COM ^c = nullptr;
	try {
		c = gcnew COM(port, BAUD);
		serial = c->ReadShort(ReadSerialNum);
		for (int i = 0; i < count && unique; i++)
			unique = getVersion_bl(ids[i]).serialnumber != serial;
		c->SendCmd(Firmware);
	} catch (COMException ^ex) {
		Dialog::CloseForm();
		ex->Show(this);
		delete[] fileName;
		EnableForm(true);
		return;
	} catch (Exception ^ex) {
		Dialog::CloseForm();
		ShowErrorMessage(L"There was an error:\n"+ex->Message, L"Error");
		EnableForm(true);
		delete[] fileName;
		return;
	} finally { if (c) c->Close(); }

	Thread::Sleep(2000); // takes about 3 seconds, so we might as well just not even check for a little bit

	// We keep checking the attached devices until a bootloader device shows up that wasn't there before
	int new_ids[MAX_ITEMS];
	int new_count;
	int id = -1;
	int tries_left = 114; // about 30 seconds max of waiting
	do {
		Thread::Sleep(250);
		new_count = getUSB2LCD_bl(new_ids, MAX_ITEMS);
	} while (((id = findDifference(unique, serial, ids, count, new_ids, new_count)) == -1) && (--tries_left > 0));

	if (id == -1) {
		Dialog::CloseForm();
		ShowErrorMessage(L"The device never showed up after reseting. Try to unplug it, close LCD Setter, plug it back in again, and start LCD Setter again.",L"Firmware Update Problem");
		delete[] fileName;
		EnableForm(true);
		return;
	}

	Dialog::SetText(L"Updating firmware...");
	Dialog::SetProgressStyle(false);
	int retval = updateFirmware(id, fileName, &progress);
	Dialog::CloseForm();
	if (retval != SUCCESS) {
		ShowErrorMessage(L"The firmware failed to update (error: "+retval+L").\nTry to unplug the device, close LCD Setter, plug the device back in again, and start LCD Setter again.",L"Firmware Update Problem");
	} else {
		ShowMessage(L"The firmware was successfully updated.\n\nYou must unplug the device and plug it back in to complete the update.\n\nSome saved settings may have changed during this process.",L"Firmware Updated Successfully",MessageBoxIcon::None);
	}
	delete[] fileName;
	EnableForm(true);
}

void Setter::InitializeComponent(void) {
	System::Windows::Forms::Label^  label2;
	System::Windows::Forms::Label^  label3;
	System::Windows::Forms::Label^  label4;
	System::Windows::Forms::Label^  label5;
	System::Windows::Forms::Label^  label6;
	System::Windows::Forms::Label^  label7;
	System::Windows::Forms::Label^  label8;
	System::Windows::Forms::Label^  label9;
	System::Windows::Forms::Label^  label10;
	System::Windows::Forms::Label^  label11;
	System::ComponentModel::ComponentResourceManager^  resources = (gcnew System::ComponentModel::ComponentResourceManager(Setter::typeid));
	this->labelPort = (gcnew System::Windows::Forms::Label());
	this->groupDisplay = (gcnew System::Windows::Forms::GroupBox());
	this->radioOnFor = (gcnew System::Windows::Forms::RadioButton());
	this->radioOnAlways = (gcnew System::Windows::Forms::RadioButton());
	this->numericOnFor = (gcnew System::Windows::Forms::NumericUpDown());
	this->checkDisplay = (gcnew System::Windows::Forms::CheckBox());
	this->checkCursor = (gcnew System::Windows::Forms::CheckBox());
	this->checkBlink = (gcnew System::Windows::Forms::CheckBox());
	this->groupScreen = (gcnew System::Windows::Forms::GroupBox());
	this->labelBacklight = (gcnew System::Windows::Forms::Label());
	this->labelContrast = (gcnew System::Windows::Forms::Label());
	this->trackBarBacklight = (gcnew System::Windows::Forms::TrackBar());
	this->trackBarContrast = (gcnew System::Windows::Forms::TrackBar());
	this->groupCustom = (gcnew System::Windows::Forms::GroupBox());
	this->customChar8 = (gcnew lcdsetter::CustomChar());
	this->customChar7 = (gcnew lcdsetter::CustomChar());
	this->customChar6 = (gcnew lcdsetter::CustomChar());
	this->customChar5 = (gcnew lcdsetter::CustomChar());
	this->customChar4 = (gcnew lcdsetter::CustomChar());
	this->customChar3 = (gcnew lcdsetter::CustomChar());
	this->customChar2 = (gcnew lcdsetter::CustomChar());
	this->customChar1 = (gcnew lcdsetter::CustomChar());
	this->groupMessage = (gcnew System::Windows::Forms::GroupBox());
	this->label = (gcnew System::Windows::Forms::TextBox());
	this->groupSpecial = (gcnew System::Windows::Forms::GroupBox());
	this->numericChar = (gcnew System::Windows::Forms::NumericUpDown());
	this->buttonInsert = (gcnew System::Windows::Forms::Button());
	this->textLine4 = (gcnew lcdsetter::LCDText());
	this->textLine3 = (gcnew lcdsetter::LCDText());
	this->textLine2 = (gcnew lcdsetter::LCDText());
	this->textLine1 = (gcnew lcdsetter::LCDText());
	this->butSetMem = (gcnew System::Windows::Forms::Button());
	this->butFirmware = (gcnew System::Windows::Forms::Button());
	this->toolTip = (gcnew System::Windows::Forms::ToolTip());
	this->butSetSerial = (gcnew System::Windows::Forms::Button());
	this->butReadMem = (gcnew System::Windows::Forms::Button());
	this->butReadCur = (gcnew System::Windows::Forms::Button());
	this->butSetCur = (gcnew System::Windows::Forms::Button());
	this->butReadButtons = (gcnew System::Windows::Forms::Button());
	this->butClear = (gcnew System::Windows::Forms::Button());
	this->butGPO = (gcnew System::Windows::Forms::Button());
	this->openFileDialog = (gcnew System::Windows::Forms::OpenFileDialog());
	label2 = (gcnew System::Windows::Forms::Label());
	label3 = (gcnew System::Windows::Forms::Label());
	label4 = (gcnew System::Windows::Forms::Label());
	label5 = (gcnew System::Windows::Forms::Label());
	label6 = (gcnew System::Windows::Forms::Label());
	label7 = (gcnew System::Windows::Forms::Label());
	label8 = (gcnew System::Windows::Forms::Label());
	label9 = (gcnew System::Windows::Forms::Label());
	label10 = (gcnew System::Windows::Forms::Label());
	label11 = (gcnew System::Windows::Forms::Label());
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->numericOnFor))->BeginInit();
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->trackBarBacklight))->BeginInit();
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->trackBarContrast))->BeginInit();
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->numericChar))->BeginInit();
	this->groupDisplay->SuspendLayout();
	this->groupScreen->SuspendLayout();
	this->groupCustom->SuspendLayout();
	this->groupMessage->SuspendLayout();
	this->groupSpecial->SuspendLayout();
	this->SuspendLayout();
	// 
	// label2
	// 
	label2->AutoSize = true;
	label2->Location = System::Drawing::Point(6, 16);
	label2->Text = L"These properties cannot be changed directly in LCD Smartie and may interfere with its display.";
	// 
	// label3
	// 
	label3->AutoSize = true;
	label3->Location = System::Drawing::Point(266, 33);
	label3->Text = L"minutes";
	// 
	// label4
	// 
	label4->AutoSize = true;
	label4->Location = System::Drawing::Point(6, 19);
	label4->Text = L"Contrast: ";
	// 
	// label5
	// 
	label5->AutoSize = true;
	label5->Location = System::Drawing::Point(8, 64);
	label5->Text = L"Backlight:";
	// 
	// label6
	// 
	label6->AutoSize = true;
	label6->Location = System::Drawing::Point(6, 22);
	label6->Text = L"Line 1:";
	// 
	// label7
	// 
	label7->AutoSize = true;
	label7->Location = System::Drawing::Point(6, 43);
	label7->Text = L"Line 2:";
	// 
	// label8
	// 
	label8->AutoSize = true;
	label8->Location = System::Drawing::Point(6, 64);
	label8->Text = L"Line 3:";
	// 
	// label9
	// 
	label9->AutoSize = true;
	label9->Location = System::Drawing::Point(6, 85);
	label9->Text = L"Line 4:";
	// 
	// label10
	// 
	label10->AutoSize = true;
	label10->Location = System::Drawing::Point(8, 21);
	label10->Text = L"Code:";
	// 
	// label11
	// 
	label11->AutoSize = true;
	label11->Location = System::Drawing::Point(6, 16);
	label11->Text = L"Click in the characters to change pixels. Right click to see the integer data.";
	// 
	// labelPort
	// 
	this->labelPort->AutoSize = true;
	this->labelPort->Location = System::Drawing::Point(12, 9);
	this->labelPort->Text = L"XXXXXXXXXXXXXXXXXXX";
	// 
	// groupDisplay
	// 
	this->groupDisplay->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left) | System::Windows::Forms::AnchorStyles::Right));
	this->groupDisplay->Controls->Add(label3);
	this->groupDisplay->Controls->Add(this->radioOnFor);
	this->groupDisplay->Controls->Add(this->radioOnAlways);
	this->groupDisplay->Controls->Add(this->numericOnFor);
	this->groupDisplay->Controls->Add(label2);
	this->groupDisplay->Controls->Add(this->checkDisplay);
	this->groupDisplay->Controls->Add(this->checkCursor);
	this->groupDisplay->Controls->Add(this->checkBlink);
	this->groupDisplay->Location = System::Drawing::Point(12, 83);
	this->groupDisplay->Size = System::Drawing::Size(495, 79);
	this->groupDisplay->TabIndex = 9;
	this->groupDisplay->TabStop = false;
	this->groupDisplay->Text = L"Display";
	// 
	// radioOnFor
	// 
	this->radioOnFor->AutoSize = true;
	this->radioOnFor->Location = System::Drawing::Point(157, 31);
	this->radioOnFor->TabIndex = 2;
	this->radioOnFor->Text = L"On for";
	this->radioOnFor->UseVisualStyleBackColor = true;
	this->radioOnFor->CheckedChanged += gcnew System::EventHandler(this, &Setter::radioOnFor_CheckedChanged);
	// 
	// radioOnAlways
	// 
	this->radioOnAlways->AutoSize = true;
	this->radioOnAlways->Checked = true;
	this->radioOnAlways->Location = System::Drawing::Point(72, 31);
	this->radioOnAlways->TabIndex = 1;
	this->radioOnAlways->TabStop = true;
	this->radioOnAlways->Text = L"Continously";
	this->radioOnAlways->UseVisualStyleBackColor = true;
	// 
	// numericOnFor
	// 
	this->numericOnFor->Enabled = false;
	this->numericOnFor->Location = System::Drawing::Point(217, 31);
	this->numericOnFor->Maximum = System::Decimal(gcnew cli::array< System::Int32 >(4) {255, 0, 0, 0});
	this->numericOnFor->Minimum = System::Decimal(gcnew cli::array< System::Int32 >(4) {1, 0, 0, 0});
	this->numericOnFor->Size = System::Drawing::Size(43, 20);
	this->numericOnFor->TabIndex = 3;
	this->numericOnFor->Value = System::Decimal(gcnew cli::array< System::Int32 >(4) {1, 0, 0, 0});
	// 
	// checkDisplay
	// 
	this->checkDisplay->AutoSize = true;
	this->checkDisplay->Checked = true;
	this->checkDisplay->CheckState = System::Windows::Forms::CheckState::Checked;
	this->checkDisplay->Location = System::Drawing::Point(6, 32);
	this->checkDisplay->TabIndex = 0;
	this->checkDisplay->Text = L"Display";
	this->checkDisplay->UseVisualStyleBackColor = true;
	this->checkDisplay->CheckedChanged += gcnew System::EventHandler(this, &Setter::checkDisplay_CheckedChanged);
	// 
	// checkCursor
	// 
	this->checkCursor->AutoSize = true;
	this->checkCursor->Location = System::Drawing::Point(6, 55);
	this->checkCursor->TabIndex = 4;
	this->checkCursor->Text = L"Cursor";
	this->checkCursor->UseVisualStyleBackColor = true;
	// 
	// checkBlink
	// 
	this->checkBlink->AutoSize = true;
	this->checkBlink->Location = System::Drawing::Point(68, 55);
	this->checkBlink->TabIndex = 5;
	this->checkBlink->Text = L"Blink";
	this->checkBlink->UseVisualStyleBackColor = true;
	// 
	// groupScreen
	// 
	this->groupScreen->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left) 
		| System::Windows::Forms::AnchorStyles::Right));
	this->groupScreen->Controls->Add(this->labelBacklight);
	this->groupScreen->Controls->Add(this->labelContrast);
	this->groupScreen->Controls->Add(this->trackBarBacklight);
	this->groupScreen->Controls->Add(label5);
	this->groupScreen->Controls->Add(label4);
	this->groupScreen->Controls->Add(this->trackBarContrast);
	this->groupScreen->Location = System::Drawing::Point(12, 168);
	this->groupScreen->Size = System::Drawing::Size(495, 111);
	this->groupScreen->TabIndex = 10;
	this->groupScreen->TabStop = false;
	this->groupScreen->Text = L"Screen";
	// 
	// labelBacklight
	// 
	this->labelBacklight->AutoSize = true;
	this->labelBacklight->Location = System::Drawing::Point(35, 86);
	this->labelBacklight->Text = L"255";
	// 
	// labelContrast
	// 
	this->labelContrast->AutoSize = true;
	this->labelContrast->Location = System::Drawing::Point(36, 41);
	this->labelContrast->Text = L"127";
	// 
	// trackBarBacklight
	// 
	this->trackBarBacklight->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left) 
		| System::Windows::Forms::AnchorStyles::Right));
	this->trackBarBacklight->LargeChange = 32;
	this->trackBarBacklight->Location = System::Drawing::Point(64, 64);
	this->trackBarBacklight->Margin = System::Windows::Forms::Padding(0);
	this->trackBarBacklight->Maximum = 255;
	this->trackBarBacklight->Size = System::Drawing::Size(422, 45);
	this->trackBarBacklight->TabIndex = 1;
	this->trackBarBacklight->TickFrequency = 4;
	this->trackBarBacklight->Value = 255;
	this->trackBarBacklight->ValueChanged += gcnew System::EventHandler(this, &Setter::trackBarBacklight_Scroll);
	this->trackBarBacklight->Scroll += gcnew System::EventHandler(this, &Setter::trackBarBacklight_Scroll);
	// 
	// trackBarContrast
	// 
	this->trackBarContrast->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left) 
		| System::Windows::Forms::AnchorStyles::Right));
	this->trackBarContrast->LargeChange = 32;
	this->trackBarContrast->Location = System::Drawing::Point(64, 19);
	this->trackBarContrast->Margin = System::Windows::Forms::Padding(0);
	this->trackBarContrast->Maximum = 255;
	this->trackBarContrast->Size = System::Drawing::Size(422, 45);
	this->trackBarContrast->TabIndex = 0;
	this->trackBarContrast->TickFrequency = 4;
	this->trackBarContrast->Value = 127;
	this->trackBarContrast->ValueChanged += gcnew System::EventHandler(this, &Setter::trackBarContrast_Scroll);
	this->trackBarContrast->Scroll += gcnew System::EventHandler(this, &Setter::trackBarContrast_Scroll);
	// 
	// groupCustom
	// 
	this->groupCustom->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left) 
		| System::Windows::Forms::AnchorStyles::Right));
	this->groupCustom->Controls->Add(label11);
	this->groupCustom->Controls->Add(this->customChar8);
	this->groupCustom->Controls->Add(this->customChar7);
	this->groupCustom->Controls->Add(this->customChar6);
	this->groupCustom->Controls->Add(this->customChar5);
	this->groupCustom->Controls->Add(this->customChar4);
	this->groupCustom->Controls->Add(this->customChar3);
	this->groupCustom->Controls->Add(this->customChar2);
	this->groupCustom->Controls->Add(this->customChar1);
	this->groupCustom->Location = System::Drawing::Point(13, 285);
	this->groupCustom->Size = System::Drawing::Size(495, 120);
	this->groupCustom->TabIndex = 11;
	this->groupCustom->TabStop = false;
	this->groupCustom->Text = L"Custom Characters";
	// 
	// customChar8
	// 
	this->customChar8->BackColor = System::Drawing::Color::White;
	this->customChar8->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar8->ForeColor = System::Drawing::Color::Black;
	this->customChar8->Location = System::Drawing::Point(404, 33);
	this->customChar8->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar8->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar8->Size = System::Drawing::Size(50, 80);
	this->customChar8->TabIndex = 7;
	// 
	// customChar7
	// 
	this->customChar7->BackColor = System::Drawing::Color::White;
	this->customChar7->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar7->ForeColor = System::Drawing::Color::Black;
	this->customChar7->Location = System::Drawing::Point(347, 33);
	this->customChar7->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar7->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar7->Size = System::Drawing::Size(50, 80);
	this->customChar7->TabIndex = 6;
	// 
	// customChar6
	// 
	this->customChar6->BackColor = System::Drawing::Color::White;
	this->customChar6->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar6->ForeColor = System::Drawing::Color::Black;
	this->customChar6->Location = System::Drawing::Point(290, 33);
	this->customChar6->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar6->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar6->Size = System::Drawing::Size(50, 80);
	this->customChar6->TabIndex = 5;
	// 
	// customChar5
	// 
	this->customChar5->BackColor = System::Drawing::Color::White;
	this->customChar5->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar5->ForeColor = System::Drawing::Color::Black;
	this->customChar5->Location = System::Drawing::Point(233, 32);
	this->customChar5->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar5->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar5->Size = System::Drawing::Size(50, 80);
	this->customChar5->TabIndex = 4;
	// 
	// customChar4
	// 
	this->customChar4->BackColor = System::Drawing::Color::White;
	this->customChar4->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar4->ForeColor = System::Drawing::Color::Black;
	this->customChar4->Location = System::Drawing::Point(176, 33);
	this->customChar4->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar4->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar4->Size = System::Drawing::Size(50, 80);
	this->customChar4->TabIndex = 3;
	// 
	// customChar3
	// 
	this->customChar3->BackColor = System::Drawing::Color::White;
	this->customChar3->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar3->ForeColor = System::Drawing::Color::Black;
	this->customChar3->Location = System::Drawing::Point(119, 33);
	this->customChar3->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar3->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar3->Size = System::Drawing::Size(50, 80);
	this->customChar3->TabIndex = 2;
	// 
	// customChar2
	// 
	this->customChar2->BackColor = System::Drawing::Color::White;
	this->customChar2->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar2->ForeColor = System::Drawing::Color::Black;
	this->customChar2->Location = System::Drawing::Point(62, 32);
	this->customChar2->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar2->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar2->Size = System::Drawing::Size(50, 80);
	this->customChar2->TabIndex = 1;
	// 
	// customChar1
	// 
	this->customChar1->BackColor = System::Drawing::Color::White;
	this->customChar1->Cursor = System::Windows::Forms::Cursors::Cross;
	this->customChar1->ForeColor = System::Drawing::Color::Black;
	this->customChar1->Location = System::Drawing::Point(6, 32);
	this->customChar1->MaximumSize = System::Drawing::Size(50, 80);
	this->customChar1->MinimumSize = System::Drawing::Size(50, 80);
	this->customChar1->Size = System::Drawing::Size(50, 80);
	this->customChar1->TabIndex = 0;
	// 
	// groupMessage
	// 
	this->groupMessage->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left) 
		| System::Windows::Forms::AnchorStyles::Right));
	this->groupMessage->Controls->Add(this->label);
	this->groupMessage->Controls->Add(this->groupSpecial);
	this->groupMessage->Controls->Add(label9);
	this->groupMessage->Controls->Add(this->textLine4);
	this->groupMessage->Controls->Add(label8);
	this->groupMessage->Controls->Add(this->textLine3);
	this->groupMessage->Controls->Add(label7);
	this->groupMessage->Controls->Add(this->textLine2);
	this->groupMessage->Controls->Add(label6);
	this->groupMessage->Controls->Add(this->textLine1);
	this->groupMessage->Location = System::Drawing::Point(13, 411);
	this->groupMessage->Size = System::Drawing::Size(495, 117);
	this->groupMessage->TabIndex = 12;
	this->groupMessage->TabStop = false;
	this->groupMessage->Text = L"Message";
	// 
	// label
	// 
	this->label->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left) 
		| System::Windows::Forms::AnchorStyles::Right));
	this->label->BorderStyle = System::Windows::Forms::BorderStyle::None;
	this->label->Location = System::Drawing::Point(383, 22);
	this->label->Multiline = true;
	this->label->ReadOnly = true;
	this->label->Size = System::Drawing::Size(105, 80);
	this->label->TabStop = false;
	this->label->Text = L"All special characters show up per the 'chars.txt' file. Every LCD has different special characters, check your manual.";
	// 
	// groupSpecial
	// 
	this->groupSpecial->Controls->Add(label10);
	this->groupSpecial->Controls->Add(this->numericChar);
	this->groupSpecial->Controls->Add(this->buttonInsert);
	this->groupSpecial->Location = System::Drawing::Point(260, 19);
	this->groupSpecial->Size = System::Drawing::Size(118, 76);
	this->groupSpecial->TabIndex = 9;
	this->groupSpecial->TabStop = false;
	this->groupSpecial->Text = L"Special Character";
	// 
	// numericChar
	// 
	this->numericChar->Location = System::Drawing::Point(60, 19);
	this->numericChar->Maximum = System::Decimal(gcnew cli::array< System::Int32 >(4) {255, 0, 0, 0});
	this->numericChar->Size = System::Drawing::Size(45, 20);
	this->numericChar->TabIndex = 0;
	this->numericChar->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &Setter::numericChar_KeyPress);
	// 
	// buttonInsert
	// 
	this->buttonInsert->Location = System::Drawing::Point(10, 45);
	this->buttonInsert->Size = System::Drawing::Size(95, 22);
	this->buttonInsert->TabIndex = 1;
	this->buttonInsert->Text = L"Type Character";
	this->buttonInsert->UseVisualStyleBackColor = true;
	this->buttonInsert->Click += gcnew System::EventHandler(this, &Setter::buttonInsert_Click);
	// 
	// textLine4
	// 
	this->textLine4->Location = System::Drawing::Point(50, 82);
	this->textLine4->Size = System::Drawing::Size(206, 20);
	this->textLine4->TabIndex = 3;
	this->textLine4->Text = L"`~!@#$^&{}\\|,.<>;;\?\"";
	this->textLine4->Enter += gcnew System::EventHandler(this, &Setter::textLine4_Enter);
	// 
	// textLine3
	// 
	this->textLine3->Location = System::Drawing::Point(50, 61);
	this->textLine3->Size = System::Drawing::Size(206, 20);
	this->textLine3->TabIndex = 2;
	this->textLine3->Text = L"0123456789-+=/*%()[]";
	this->textLine3->Enter += gcnew System::EventHandler(this, &Setter::textLine3_Enter);
	// 
	// textLine2
	// 
	this->textLine2->Location = System::Drawing::Point(50, 40);
	this->textLine2->Size = System::Drawing::Size(206, 20);
	this->textLine2->TabIndex = 1;
	this->textLine2->Text = L"abcdefghijklmnopqrst";
	this->textLine2->Enter += gcnew System::EventHandler(this, &Setter::textLine2_Enter);
	// 
	// textLine1
	// 
	this->textLine1->Location = System::Drawing::Point(50, 19);
	this->textLine1->Size = System::Drawing::Size(206, 20);
	this->textLine1->TabIndex = 0;
	this->textLine1->Text = L"ABCDEFGHIJKLMNOPQRST";
	this->textLine1->Enter += gcnew System::EventHandler(this, &Setter::textLine1_Enter);
	// 
	// butSetMem
	// 
	this->butSetMem->Location = System::Drawing::Point(131, 54);
	this->butSetMem->Size = System::Drawing::Size(112, 23);
	this->butSetMem->TabIndex = 6;
	this->butSetMem->Text = L"Set LCD Memory";
	this->toolTip->SetToolTip(this->butSetMem, L"Set the saved LCD values from this form.\r\nThis will be used at bootup before anything has a chance to communicate with the device.\r\nNote: This inadvertently sets many current values.");
	this->butSetMem->UseVisualStyleBackColor = true;
	this->butSetMem->Click += gcnew System::EventHandler(this, &Setter::butSetMem_Click);
	// 
	// butFirmware
	// 
	this->butFirmware->Location = System::Drawing::Point(367, 26);
	this->butFirmware->Size = System::Drawing::Size(112, 23);
	this->butFirmware->TabIndex = 4;
	this->butFirmware->Text = L"Update Firmware";
	this->toolTip->SetToolTip(this->butFirmware, L"Update the firmware of the USB2LCD+");
	this->butFirmware->UseVisualStyleBackColor = true;
	this->butFirmware->Click += gcnew System::EventHandler(this, &Setter::butFirmware_Click);
	// 
	// toolTip
	// 
	this->toolTip->AutomaticDelay = 100;
	this->toolTip->AutoPopDelay = 2000;
	this->toolTip->InitialDelay = 100;
	this->toolTip->ReshowDelay = 20;
	// 
	// butSetSerial
	// 
	this->butSetSerial->Location = System::Drawing::Point(249, 4);
	this->butSetSerial->Size = System::Drawing::Size(112, 20);
	this->butSetSerial->TabIndex = 13;
	this->butSetSerial->Text = L"Set ID";
	this->toolTip->SetToolTip(this->butSetSerial, L"Sets the ID of the device");
	this->butSetSerial->UseVisualStyleBackColor = true;
	this->butSetSerial->Click += gcnew System::EventHandler(this, &Setter::butSetSerial_Click);
	// 
	// butReadMem
	// 
	this->butReadMem->Location = System::Drawing::Point(13, 54);
	this->butReadMem->Size = System::Drawing::Size(112, 23);
	this->butReadMem->TabIndex = 5;
	this->butReadMem->Text = L"Read LCD Memory";
	this->toolTip->SetToolTip(this->butReadMem, L"Read the saved LCD values into the form");
	this->butReadMem->UseVisualStyleBackColor = true;
	this->butReadMem->Click += gcnew System::EventHandler(this, &Setter::butReadMem_Click);
	// 
	// butReadCur
	// 
	this->butReadCur->Location = System::Drawing::Point(13, 26);
	this->butReadCur->Size = System::Drawing::Size(112, 23);
	this->butReadCur->TabIndex = 1;
	this->butReadCur->Text = L"Read LCD Current";
	this->toolTip->SetToolTip(this->butReadCur, L"Reads the current LCD values into the form");
	this->butReadCur->UseVisualStyleBackColor = true;
	this->butReadCur->Click += gcnew System::EventHandler(this, &Setter::butReadCur_Click);
	// 
	// butSetCur
	// 
	this->butSetCur->Location = System::Drawing::Point(131, 26);
	this->butSetCur->Size = System::Drawing::Size(112, 23);
	this->butSetCur->TabIndex = 2;
	this->butSetCur->Text = L"Set LCD Current";
	this->toolTip->SetToolTip(this->butSetCur, L"Set the current LCD values from this form.\r\nLike using any other program (such as LCD Smartie)");
	this->butSetCur->UseVisualStyleBackColor = true;
	this->butSetCur->Click += gcnew System::EventHandler(this, &Setter::butSetCur_Click);
	// 
	// butReadButtons
	// 
	this->butReadButtons->Location = System::Drawing::Point(249, 54);
	this->butReadButtons->Size = System::Drawing::Size(112, 23);
	this->butReadButtons->TabIndex = 7;
	this->butReadButtons->Text = L"Read Buttons";
	this->toolTip->SetToolTip(this->butReadButtons, L"Reads the button states");
	this->butReadButtons->UseVisualStyleBackColor = true;
	this->butReadButtons->Click += gcnew System::EventHandler(this, &Setter::butReadButtons_Click);
	// 
	// butClear
	// 
	this->butClear->Location = System::Drawing::Point(249, 26);
	this->butClear->Size = System::Drawing::Size(112, 23);
	this->butClear->TabIndex = 3;
	this->butClear->Text = L"Clear Display";
	this->toolTip->SetToolTip(this->butClear, L"Clears the LCD display");
	this->butClear->UseVisualStyleBackColor = true;
	this->butClear->Click += gcnew System::EventHandler(this, &Setter::butClear_Click);
	// 
	// butGPO
	// 
	this->butGPO->Location = System::Drawing::Point(367, 54);
	this->butGPO->Size = System::Drawing::Size(112, 23);
	this->butGPO->TabIndex = 8;
	this->butGPO->Text = L"Edit GPOs";
	this->butGPO->UseVisualStyleBackColor = true;
	this->butGPO->Click += gcnew System::EventHandler(this, &Setter::butGPO_Click);
	// 
	// openFileDialog
	// 
	this->openFileDialog->Filter = L"HEX File (*.hex)|*.hex";
	this->openFileDialog->RestoreDirectory = true;
	// 
	// Setter
	// 
	this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
	this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
	this->ClientSize = System::Drawing::Size(519, 540);
	this->Controls->Add(this->butSetSerial);
	this->Controls->Add(this->butFirmware);
	this->Controls->Add(this->butGPO);
	this->Controls->Add(this->butClear);
	this->Controls->Add(this->butReadButtons);
	this->Controls->Add(this->butSetCur);
	this->Controls->Add(this->butReadCur);
	this->Controls->Add(this->butReadMem);
	this->Controls->Add(this->groupMessage);
	this->Controls->Add(this->groupCustom);
	this->Controls->Add(this->groupScreen);
	this->Controls->Add(this->groupDisplay);
	this->Controls->Add(this->labelPort);
	this->Controls->Add(this->butSetMem);
	this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
	this->Icon = (cli::safe_cast<System::Drawing::Icon^  >(resources->GetObject(L"$this.Icon")));
	this->MaximizeBox = false;
	this->Name = L"Setter";
	this->Text = L"LCD Setter";
	this->FormClosing += gcnew System::Windows::Forms::FormClosingEventHandler(this, &Setter::Setter_FormClosing);
	this->groupDisplay->ResumeLayout();
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->numericOnFor))->EndInit();
	this->groupScreen->ResumeLayout();
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->trackBarBacklight))->EndInit();
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->trackBarContrast))->EndInit();
	this->groupCustom->ResumeLayout();
	this->groupMessage->ResumeLayout();
	this->groupSpecial->ResumeLayout();
	(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->numericChar))->EndInit();
	this->ResumeLayout();
}