// usb2lcd-util.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"
#include "usb2lcd-util.h"
#include "mpusbapi.h"

#define PIPE_NAME L"\\MCHP_EP1"

/////////////////////////////////////////////// Bootloader ////////////////////////////////////

#define SENDRCV_FAILED			0
#define SENDRCV_SUCCESS			1
#define SENDRCV_SUCCESS_SHORT	2 // success, but recieved data is short of expected

// from boot.h in the bootloader
#define READ_VERSION	0x00
#define WRITE_FLASH     0x02
//#define RESET			0xFF

DWORD SendReceivePacket(HANDLE in, HANDLE out, BYTE *send, DWORD sendLen, BYTE *rcv, DWORD *rcvLen, UINT sendDelay, UINT rcvDelay) {
    DWORD sentLen = 0;
    DWORD expRcvLength = *rcvLen;
    if(MPUSBWrite(out, send, sendLen, &sentLen, sendDelay) && MPUSBRead(in, rcv, expRcvLength, rcvLen, rcvDelay)) {
        if(*rcvLen == expRcvLength) {
            return SENDRCV_SUCCESS;
        } else if(*rcvLen < expRcvLength) {
            return SENDRCV_SUCCESS_SHORT;
        }
    }
    return SENDRCV_FAILED;
}

VersionInfo getVersion(HANDLE in, HANDLE out) {
	BYTE rcv[6], send[2] = {READ_VERSION, 2}; //command, length
    DWORD len = 6;
	VersionInfo v;
	v._all = COMMUNICATION_FAILED;
	if (SendReceivePacket(in, out, send, 2, rcv, &len, 1000, 1000) == SENDRCV_SUCCESS && len == 6 && rcv[0] == READ_VERSION && rcv[1] == 0x02) {
		v.serialnumber = rcv[2] | rcv[3] << 8;
		v.module = rcv[4];
		v.version = rcv[5];
    }
	return v;
}

VersionInfo getVersion_bl(int id) {
	HANDLE in = MPUSBOpen(id, USB_BL_ID, PIPE_NAME, MP_READ, 0);
	HANDLE out = MPUSBOpen(id, USB_BL_ID, PIPE_NAME, MP_WRITE, 0);
	VersionInfo v;
	v._all = COULD_NOT_OPEN_DEVICE;
	if (in != INVALID_HANDLE_VALUE && out != INVALID_HANDLE_VALUE) {
		v = getVersion(in, out);
		MPUSBClose(in);
		MPUSBClose(out);
	}
	return v;
}

int sendData(BYTE *data, DWORD len, DWORD addr, PROGRESS_CB progress, HANDLE in, HANDLE out) {
	if (addr % 32 > 0) {
		return ADDR_MUST_BE_MULT_32;
	}
	DWORD i, rLen = 7, len_64 = len / 64 + ((len % 64) ? 1 : 0);
	int x = 0;
	BYTE rcv[7], send[64];
	send[0] = WRITE_FLASH;
	send[1] = 7;
	send[2] = addr & 0xFF; /* low */
	send[3] = (addr >> 8) & 0xFF; /* high */
	send[4] = (addr >> 16) & 0x1F; /* upper */
	send[5] = len_64 & 0xFF;
	send[6] = (len_64 >> 8) & 0xFF;
	if (SendReceivePacket(in, out, send, 7, rcv, &rLen, 1000, 1000) == SENDRCV_FAILED) { return COMMUNICATION_FAILED; }

	for (i = 0, x = 0; i < len; i++, x++) {
		send[x] = data[i];
		if (x == 63) {
			x = -1;
			rLen = 7;
			if (SendReceivePacket(in, out, send, 64, rcv, &rLen, 1000, 1000) == SENDRCV_FAILED) { return COMMUNICATION_FAILED; }
			/*addr = (rcv[2] << 0) | (rcv[3] << 8) | (rcv[4] << 16);
			len_64 = (rcv[5] << 0) | (rcv[6] << 8);
			printf("a: %04x  c: %3d  ", addr, len_64);
			for (int j = 0; j < 64; j++) {
				printf("%02X", send[j]);
			}
			printf("\n");*/
			progress(i, len);
		}
	}
	if (x != 0) {
		memset(send+x, FILLER_BYTE, 64-x);
		rLen = 7;
		if (SendReceivePacket(in, out, send, 64, rcv, &rLen, 1000, 1000) == SENDRCV_FAILED) { return COMMUNICATION_FAILED; }
	}
	progress(len, len);
	return SUCCESS;
}

int sendData(BYTE *data, DWORD len, DWORD addr, PROGRESS_CB progress, int id) {
	HANDLE in = MPUSBOpen(id, USB_BL_ID, PIPE_NAME, MP_READ, 0);
	HANDLE out = MPUSBOpen(id, USB_BL_ID, PIPE_NAME, MP_WRITE, 0);
	if (in != INVALID_HANDLE_VALUE && out != INVALID_HANDLE_VALUE) {
		int x = sendData(data, len, addr, progress, in, out);
		MPUSBClose(in);
		MPUSBClose(out);
		return x;
	}
	return COULD_NOT_OPEN_DEVICE;
}

/*bool reset(HANDLE out) {
	BYTE send[2] = {RESET, 2};
    DWORD sentLen = 0;
	return MPUSBWrite(out, send, 2, &sentLen, 1000) ? true : false;
}

bool reset_bl(int id) {
	HANDLE out = MPUSBOpen(id, USB_BL_ID, PIPE_NAME, MP_WRITE, 0);
	if (out != INVALID_HANDLE_VALUE) {
		bool b = reset(out);
		MPUSBClose(out);
		return b;
	}
	return false;
}*/

#define HEX_T_DATA	0x00
#define HEX_T_EOF	0x01
#define HEX_T_EXT	0x04

#define sumBYTE(x) (x & 0xFF)
#define sumWORD(x) ((x & 0xFF) + ((x >> 8) & 0xFF))

#define MAX_ADDR		0x7D00
#define MAX_SIZE		(MAX_ADDR-STARTING_POINT)

typedef unsigned int BYTE_;
typedef unsigned int WORD_;

/**
Reads a HEX file into buffer, marking the bytes written in mask.
len is the maximum length that can be written to buffer or buMask.
file is the filename to read.
Returns COULD_NOT_OPEN_FILE or CHECKSUM_FAILED in case of problems reading the file.
Any other negative value indicates the buffers were to small to fit the data and the magnitude of the value is how many bytes are required.
A positibve indicates how many bytes were written to the buffers.
The first byte written to the buffer is actually STARTING_POINT, not 0. All bytes in the HEX file before STARTING_POINT are ignored.
This will ignore all EEPROM bytes and CONFIG bits from the HEX file.
**/
long readHEXfile(BYTE *buffer, bool *buMask, DWORD len, TCHAR *file) {
	FILE *f = _tfopen(file, TEXT("r"));
	if (!f) {
		return COULD_NOT_OPEN_FILE;
	}
	
	//:BBAAAATTHHHH...HHHHCC
	BYTE data[MAX_ADDR];
	bool mask[MAX_ADDR];
	memset(mask, false, MAX_ADDR);
	BYTE_ b; // number of bytes in h row, max 0x10 (16)
	WORD_ a; // address
	BYTE_ t; // type of entry (HEX_T_XXX)
	BYTE_ h[16]; // data, 16 bytes max
	BYTE_ c; // checksum (two's complement of sum of all the other bytes)
	
	WORD_ max_a = STARTING_POINT;

	while (!feof(f)) {
		BYTE c_ = 0;
		if (_ftscanf(f, TEXT(":%2x%4x%2x"), &b, &a, &t) != 3) {
			fclose(f);
			return NOT_HEX_FORMAT;
		}
		c_ = sumBYTE(b) + sumWORD(a) + sumBYTE(t);
		for (BYTE_ i = 0; i < b; i++) {
			if (_ftscanf(f, TEXT("%2x"), h+i) != 1) {
				fclose(f);
				return NOT_HEX_FORMAT;
			}
			c_ += sumBYTE(h[i]);
		}
		if (_ftscanf(f, TEXT("%2x\n"), &c) != 1) {
			fclose(f);
			return NOT_HEX_FORMAT;
		}
		if (c + c_ != 0x100 && !(c == 0 && c_ == 0)) {
			fclose(f);
			return CHECKSUM_FAILED;
		}
		if (t == HEX_T_EXT) {
			// skip
		} else if (t == HEX_T_EOF) {
			break; // done with file
		} else if (t == HEX_T_DATA) {
			// data byte, only care about > the starting point
			if (a >= STARTING_POINT) {
				if (a+b > max_a) { max_a = a+b; }
				for (WORD_ i = 0; i < b; i++) {
					data[a+i] = (BYTE)h[i];
					mask[a+i] = true;
				}
			}
		} // else unknown, do nothing
	}

	if (len < max_a - STARTING_POINT) {
		return STARTING_POINT - max_a;
	}

	for (WORD_ i = STARTING_POINT; i < max_a; i++) {
		if (mask[i])
			buffer[i - STARTING_POINT] = data[i];
		buMask[i - STARTING_POINT] = mask[i];
	}

	fclose(f);
	return max_a - STARTING_POINT;
}

/*
Does the exact same thing as readHEXfile except that none of the data is saved.
It essentially checks that the file can be read, is in the HEX format, and all the checksums work.
If all checks pass, it returns the requried size of the buffers in readHEXfile.
*/
long checkHEXfile(TCHAR *file) {
	FILE *f = _tfopen(file, TEXT("r"));
	if (!f) { return COULD_NOT_OPEN_FILE; }
	BYTE_ b, t, c, h[16];
	WORD_ a; // address
	WORD_ max_a = STARTING_POINT;
	while (!feof(f)) {
		BYTE c_ = 0;
		if (_ftscanf(f, TEXT(":%2x%4x%2x"), &b, &a, &t) != 3) { fclose(f); return NOT_HEX_FORMAT; }
		c_ = sumBYTE(b) + sumWORD(a) + sumBYTE(t);
		for (BYTE_ i = 0; i < b; i++) {
			if (_ftscanf(f, TEXT("%2x"), h+i) != 1) { fclose(f); return NOT_HEX_FORMAT; }
			c_ += sumBYTE(h[i]);
		}
		if (_ftscanf(f, TEXT("%2x\n"), &c) != 1) { fclose(f); return NOT_HEX_FORMAT; }
		if (c + c_ != 0x100 && !(c == 0 && c_ == 0)) { fclose(f); return CHECKSUM_FAILED; }
		if (t == HEX_T_EOF) break;
		if (t == HEX_T_DATA && a+b > max_a) { max_a = a+b; }
	}
	fclose(f);
	return max_a - STARTING_POINT;
}

int updateFirmware(int id, TCHAR *file, PROGRESS_CB progress) {
	BYTE d[MAX_SIZE];
	bool m[MAX_SIZE];
	memset(d, FILLER_BYTE, MAX_SIZE);
	memset(m, false, MAX_SIZE);
	long retval = readHEXfile(d, m, MAX_SIZE, file);
	if (retval < 0) {
		if (retval == COULD_NOT_OPEN_FILE || retval == CHECKSUM_FAILED)
			return retval;
		// we didn't give it enough room? what?
		return BUFFER_OVERFLOW;
	}
	HANDLE in = MPUSBOpen(id, USB_BL_ID, PIPE_NAME, MP_READ, 0);
	HANDLE out = MPUSBOpen(id, USB_BL_ID, PIPE_NAME, MP_WRITE, 0);
	if (in != INVALID_HANDLE_VALUE && out != INVALID_HANDLE_VALUE) {
		int x = sendData(d, retval, STARTING_POINT, progress, in, out);
		/*bool b = false;
		if (x == SUCCESS) {
			b = reset(out);
		}*/
		MPUSBClose(in);
		MPUSBClose(out);
		return x != SUCCESS ? x : SUCCESS; //(b ? SUCCESS : RESET_FAILED);
	}
	return COULD_NOT_OPEN_DEVICE;
}

/**
Finds a USB2LCD that is in boot-loader mode.
Returns an ID upon finding it, otherwise NOT_FOUND.
**/
int getUSB2LCD_bl(int *ids, int size) {
	DWORD max_count = MPUSBGetDeviceCount(USB_BL_ID), count = 0;
	int pos = 0;
	VersionInfo v;
	for (DWORD i = 0; i < MAX_NUM_MPUSB_DEV && count < max_count && pos >= 0; i++) {
		v = getVersion_bl(i);
		if (v._all != COULD_NOT_OPEN_DEVICE) {
			if (v.module == 0x5B) {
				if (pos < size)
					ids[pos++] = i;
				else
					pos = BUFFER_OVERFLOW;
			}
			count++;
		}
	}
	return pos;
}


///////////////////// COM /////////////////////////////////////////

#define MAX_KEY_LEN	256
#define BUF_SIZE	1024

static int matchExists(HKEY keyCom, wchar_t *serv, wchar_t *port) {
	DWORD err;
	wchar_t name[BUF_SIZE], data[BUF_SIZE];
	DWORD name_len, data_len;
	DWORD type;
	for (DWORD j = 0; ; ++j) {
		name_len = BUF_SIZE;
		data_len = BUF_SIZE*sizeof(wchar_t);
		err = RegEnumValue(keyCom, j, name, &name_len, NULL, &type, (LPBYTE)data, &data_len);
		if (err == ERROR_NO_MORE_ITEMS) { return 0; } // not found
		if (err != ERROR_SUCCESS) { return ERROR_REG_QUERY_VALUE; }
		if (_wcsicmp(data, port) == 0) {
			// found the port
			if (_wcsnicmp(name, data, swprintf(data, BUF_SIZE, L"\\Device\\%s", serv)) == 0) {
				// and it belongs to the right service
				return _wtoi(port+3);
			}
			return 0; // doesn't belong to the right service
		}
	}
}

int getUSB2LCD_com(int *ids, int size) {
	DWORD err;
	HKEY keyUsb, keyCom, keyDev, keyParam;

	wchar_t key_name[BUF_SIZE];

	wchar_t id[MAX_KEY_LEN], serv[BUF_SIZE], port[BUF_SIZE];
	DWORD id_len, serv_len, port_len;
	int port_num;

	DWORD type = 0;
	int cur = 0;
	
	// Get the key containing all of the USB2LCD+ devices
	swprintf(key_name, BUF_SIZE, L"SYSTEM\\CurrentControlSet\\Enum\\USB\\%s", USB_COM_ID);
	if ((err = RegOpenKeyEx(HKEY_LOCAL_MACHINE, key_name, 0, KEY_ENUMERATE_SUB_KEYS, &keyUsb)) != ERROR_SUCCESS) {
		return err == ERROR_FILE_NOT_FOUND ? ERROR_NO_DEVICES : ERROR_REG_OPEN_KEY;
	}

	// Get the listing of current serial devices
	if ((err = RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"HARDWARE\\DEVICEMAP\\SERIALCOMM", 0, KEY_QUERY_VALUE, &keyCom)) != ERROR_SUCCESS) {
		RegCloseKey(keyUsb);
		return err == ERROR_FILE_NOT_FOUND ? ERROR_NO_COM_PORTS : ERROR_REG_OPEN_KEY;
	}

	// Go through each device in the list
	for (DWORD i = 0; ; ++i) {
		// Get the next device
		id_len = MAX_KEY_LEN;
		err = RegEnumKeyEx(keyUsb, i, id, &id_len, NULL, NULL, NULL, NULL);
		if (err == ERROR_NO_MORE_ITEMS) { break; }
		if (err) { /*error!*/ break; }

		// Get the service name
		serv_len = BUF_SIZE*sizeof(wchar_t);
		if ((err = RegOpenKeyEx(keyUsb, id, 0, KEY_QUERY_VALUE, &keyDev)) != ERROR_SUCCESS) { /*error!*/ continue; }
		if ((err = RegQueryValueEx(keyDev, L"Service", NULL, &type, (LPBYTE)serv, &serv_len)) != ERROR_SUCCESS || type != REG_SZ) { /*error!*/ RegCloseKey(keyDev); continue; }

		// Get the port name
		port_len = BUF_SIZE;
		if ((err = RegOpenKeyEx(keyDev, L"Device Parameters", 0, KEY_QUERY_VALUE, &keyParam)) != ERROR_SUCCESS) { /*error!*/ RegCloseKey(keyDev); continue; }
		err = RegQueryValueEx(keyParam, L"PortName", NULL, &type, (LPBYTE)port, &port_len);
		
		// Close keys opened so far
		RegCloseKey(keyParam);
		RegCloseKey(keyDev);

		if (err != ERROR_SUCCESS || type != REG_SZ) { /*error!*/ continue; }

		// See if that port is actually active and associated with the service
		port_num = matchExists(keyCom, serv, port);
		if (port_num > 0) {
			// Add it to the list
			if (cur == size) { cur = BUFFER_OVERFLOW; break; }
			ids[cur++] = port_num;
		} else if (port_num < 0) {
			err = port_num;
			/* error! */
			continue;
		}
	}
	RegCloseKey(keyCom);
	RegCloseKey(keyUsb);
	return cur;
}
